@@ -4,13 +4,15 @@ import test_runner
 
 if __name__ == '__main__':
     modules = [
-        'parsing',
-        'destinations',
-        'lifecycle',
-        'transactions',
         'ack',
+        'destinations',
         'errors',
+        'lifecycle',
+        'parsing',
+        'queue_properties',
+        'redelivered',
         'reliability',
+        'transactions',
     ]
     test_runner.run_unittests(modules)
 
